-------------------------------------------------------------- 
--  Subject : DAO. Demo program
--  Copyright (c) 2000 by A.Baranovsky
--------------------------------------------------------------

/*Group Object, Groups Collection, User Object, and Users Collection Example

This example illustrates the use of the Group and User objects and the Groups 
and Users collections. First, it creates a new User object and appends the 
object to the Users collection of the default Workspace object. Next, it 
creates a new Group object and appends the object to the Groups collection 
of the default Workspace object. Then the example adds user Pat Smith to the 
Accounting group. Finally, it enumerates the Users and Groups collections of 
the default Workspace object. See the methods and properties listed in the 
Group and User summary topics for additional examples.*/

uses com, dao, daoworks, daoengin, daouser

CoInitialize()

object DE : DBEngine
object wrkDefault : Workspace
object U, usrNew : User
object G, grpNew, grpMember : Group
object Users, Groups : Collection

try

  DE.Init( 'DAO.DBEngine.36' )
  wrkDefault := DE[ 'Workspaces' ][0]

// Create and append new user.

  usrNew := WrkDefault.CreateUser( 'Pat Smith' )
  usrNew.PutPID( 'abc123DEF456' )
  usrNew.PutPassword( 'Password1' )

  Users := WrkDefault[ 'Users' ]
  Users.Append( usrNew )

// Create and append new group

  grpNew := WrkDefault.CreateGroup( 'Accounting' )
  grpNew.PutPID( 'UVW987xyz654' )

  Groups := WrkDefault[ 'Groups' ]
  Groups.Append( grpNew )

/* Make the user Pat Smith a member of the Accounting group by creating and 
   adding the appropriate Group object to the user's Groups collection. The 
   same is accomplished if a User object representing Pat Smith is created 
   and appended to the Accounting group's Users collection. */

   grpMember := usrNew.CreateGroup( 'Accounting' )

   Groups := usrNew[ 'Groups' ]
   Groups.Append( grpMember )

   writeln( 'Users collection:' )

// Enumerate all User objects in the default workspace's Users collection.

   for I:=0 to Users.Count() - 1 do
     U := Users[ I ] 
     writeln( U.GetName() )
     writeln( 'Belongs to these groups:' )

//   Enumerate all Group objects in each User object's Groups collection.
      
     Groups := U[ 'Groups' ]
     if Groups.Count() <> 0 then
       for J:=0 to Groups.Count() - 1 do
         G := Groups[ J ]
         writeln( G.GetName() )
       end
     else
       writeln( 'None' )
     end
   end

   writeln( 'Groups collection:' )

// Enumerate all Group objects in the default workspace's Groups collection.

   Groups := WrkDefault[ 'Groups' ]
   for I:=0 to Groups.Count() - 1 do
     G := Groups[ I ]
     writeln( G.GetName() )
     writeln( 'Has as its members:' )

//   Enumerate all User objects in each Group object's Users collection.

     Users := G[ 'Users' ]

     if Users.Count() <> 0 then
       for J:=0 to Users.Count() - 1 do
         U := Users[ I ]
         writeln( U.GetName() )
       end
     else
       writeln( 'None' )
     end
   end 

// Delete new User and Group objects because this is only a demonstration.

  Users := WrkDefault[ 'Users' ]
  Users.Delete( 'Pat Smith' )

  Groups := WrkDefault[ 'Groups' ]
  Groups.Delete( 'Accounting' )

finally
  CoUninitialize()
  writeln('')
  writeln( 'COM have closed' )
end

return
