--------------------------------------------------------------  
--  Subject : Linear algebra
--  Copyright (c) 1999 by A.Baranovsky
--------------------------------------------------------------

class Matrix
  function Create( N, M )
  function Determinant( A )
  function Mult( A, B )
  function Add( A, B )
  function Scalar( A, L )
  function Invert( A, var S )
  function Print( A )
end

implementation Matrix

  function Create( N, M )
    A := array( N )
    for I:=1 to N do
      if M > 1 then
        A[ I ] := array( M )
        for J:=1 to M do
          A[ I, J ] := 0.0
        end
      else
        A[ I ] := 0.0 
      end
    end
  return A

  function Determinant( A )
    N := Card( A )
    D := 1.0
    for K := 1 to N do
      Maximum := 0.0
      for I := K to N do
        T := A[ I, K ]
        if Abs( T ) > Abs( Maximum ) then
          Maximum := T
          J := I
        end
      end
      if Maximum = 0.0 then
        D := 0.0
        goto Fin
      end
      if J <> K then
        D := - D
        for I:=K to N do
          Swap( A[ J, I ], A[ K, I ] )
        end
      end
      for I:= K + 1 to N do
        T := A[ I, K ] / Maximum
        for J:= K + 1 to N do
          A[ I, J ] := A[ I, J ] - T * A[ K, J ]
        end
      end
      D := D * A[ K, K ]
    end
    label Fin
  return D

  function Mult( A, B )
    N := Card( A )
    M := Card( B[1] )
    L := Card( A[1] )

    C := array( N )
    for I:=1 to N do
      C[ I ] := array( M )
      for J:=1 to M do
        CIJ := 0.0
        for K :=1 to L do
          CIJ := CIJ + A[ I, K ] * B[ K, J ]
        end
        C[ I, J ] := CIJ
      end
    end
  return C

  function Add( A, B )
    N := Card( A )
    M := Card( A[1] )
    C := array( N )
    for I:=1 to N do
      C[ I ] := array( M ) 
      for J:=1 to M do
        C[ I, J ] := A[ I, J ] + B[ I, J ]
      end
    end
  return C

  function Scalar( A, L )
    C := A
    for I:=1 to Card( C ) do
      for J:=1 to Card( C[ I ] ) do
        C[ I, J ] := A[ I, J ] * L
      end
    end
  return C

  function Invert( A, var S )

  -- Initialization

    N := Card( A )

    M := N + N

    AA := array( N )
    for I:=1 to N do
      AA[ I ] := array( M )
    end

    S := 0
    for I:=1 to N do
      for J:=1 to M do
        if    J <= N then
          AA[ I, J ] := A[ I, J ]
        elsif J = N + I then
          AA[ I, J ] := 1.0
        else
          AA[ I, J ] := 0.0
        end
      end
    end

  -- Start

    for I:=1 to N do
      K := I
      label test0
      if AA[ K, I ] = 0.0 then
        S := 1
        if K < N then
          Inc( K )
        else
          goto Fin
        end
        goto test0
      end
      if S = 1 then
        for J:=1 to M do
          T := AA[ K, J ]
          AA[ K, J ] := AA[ I, J ]
          AA[ I, J ] := T
        end
      end

      J := M
      while J >= I do
        AA[ I, J ] := AA[ I, J ] / AA[ I, I ]
        Dec( J )
      end

      for K:=1 to N do
        if K <> I then
          J := M
          while J >= I do
            AA[ K, J ] := AA[ K, J ] - AA[ I, J ] * AA[ K, I ]
            Dec( J )
          end
        end
      end
    end

    C := A

    for I:=1 to N do
      for J:=1 to N do
        C[ I, J ] := AA[ I, J + N ]
      end
    end

    S := 0

    label Fin

  return C

  function Print( A )
    for I:=1 to Card( A ) do
      writeln( A[ I ] )
    end
  return 0
end

----- Main program ------------------

A := array( 2 )
A[ 1 ] := [ 1.2, 2.3, 3.4 ]
A[ 2 ] := [ 3.2, 4.0, 5.1 ]

B := array( 3 )
B[ 1 ] := [ 1.2,   2.2,  3.3,  4.4,  5.1 ]
B[ 2 ] := [ 6.3,   7.4,  8.5,  9.3, 10.1 ]
B[ 3 ] := [ 11.0, 12.5, 13.7, 14.0, 15.9 ]

C := Matrix.Mult( A, B )
Matrix.Print( C )

C := Matrix.Add( A, A )
Matrix.Print( C )

C := Matrix.Scalar( A, 2.0 )
Matrix.Print( C )

D := Matrix.Determinant( C )
writeln( D )

-- Create Hilbert matrix  

  N := 4
  A := array( N )

  for I:=1 to N do
    A[ I ] := array( N )
    for J:=1 to N do
      A[ I, J ] := 1.0 / ( I + J - 1.0 )
    end
  end

-- Invert it

  C := Matrix.Invert( A, S )
  Matrix.Print( C )

return 