--------------------------------------------------------------  
--  Subject : Linear programming, Simplex algorithm
--  Copyright (c) 1999 by A.Baranovsky
--------------------------------------------------------------

class Simplex
  function Init( InitA, InitB, InitC )
  function Mark()
  function FindElem()
  function ReCalc()
  function Run()
  function Output()
end

implementation Simplex

  local A, B, C, D, N, M, L, K, Code

  function Init( InitA, InitB, InitC )
    A := InitA
    B := InitB
    C := InitC
    M := Card( A )
    N := Card( A[1] )
    D := array( N )
  return 0

  function Mark()
    for J:=1 to N do
      D[ J ] := 0.0
      for I:=1 to M do
        K := B[ I ]
        D[ J ] := D[ J ] + C[ K ] * A[ I, J ]
      end
      if J < N then
        D[ J ] := D[ J ] - C[ J ]
      else
        D[ J ] := D[ J ] + C[ J ]
      end
    end
  return 0

  function FindElem()
    K := 0
    S := 0.0
    for J:=1 to N - 1 do
      if D[ J ] < S then
        K := J
        S := D[ J ]
      end
    end
    L := 0
    S := 1000000000
    if K > 0 then
      for I:=1 to M do
        if ( A[ I, K ] > 0 ) and ( S > A[ I, N ] / A[ I, K ] ) then
          L := I
          S := A[ I, N ] / A[ I, K ]
        end
      end
    end
  return 0

  function ReCalc()
    B[ L ] := K
    Div := A[ L, K ]
    for I:=1 to M do
      for J :=1 to N do
        if ( J <> K ) and ( I <> L ) then
          A[ I, J ] := A[ I, J ] - A[ L, J ] * A[ I, K ] / Div
        end
      end
    end
    for J:=1 to N do
      A[ L, J ] := A[ L, J ] / Div
    end
    for I:=1 to M do
      if I <> L then
        A[ I, K ] := 0.0
      end
    end
  return 0

  function Run()
    I := 0
    R := false
    repeat
      Inc( I )
      Mark()
      FindElem()
      if    ( K > 0 ) and ( L > 0 ) then
        ReCalc()
      elsif ( K > 0 ) and ( L = 0 ) then
        R    := true
        Code := 1
      elsif K = 0 then
        R    := true
        Code := 0
      end
    until R
  return Code

  function Output()
    if Code = 0 then
      for I:=1 to M do
        writeln( 'X'+STR(B[ I ])+'  ='+ STR(A[ I, N ]) )
      end
      writeln( D[ N ] )
    else
      writeln( 'Solution is absent' )
    end
  return 0
end

-------- Main program --------------------

A := array( 3 )

A[1] := [  4.0,  5.0, 1.0, 0.0, 0.0, 61.0 ] 
A[2] := [ -3.0,  4.0, 0.0, 1.0, 0.0, 24.0 ]
A[3] := [  5.0, -3.0, 0.0, 0.0, 1.0, 30.0 ]

C := [ 6.0, 2.0, 0.0, 0.0, 0.0, 0.0 ]

object S : Simplex
S.Init( A, [ 3, 4, 5 ], C )
S.Run()
S.Output()

return

